using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Xml.Serialization;


namespace CompanyX.TechnicalArchitecture
{
    /// <summary>
    /// Manager for serialization
    /// </summary>
    public sealed class SerializationManager
    {
        private SerializationManager() { }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="objX">object to be serialized.</param>
        /// <param name="useBinary">Flag to indicate whether object should be serialized to xml or binary.
        /// if ( true, then will serialize to binary, else, will serialize to xml.</param>
        /// <returns>Serialized object</returns>
        public static string Serialize(object objX, bool useBinary)
        {
            string retVal = "";

            if (useBinary)
            {
                // First serialize the object into byte array
                byte[] btarr = SerializeToByteArray(objX);

                //Base64 encode byte array
                retVal = Convert.ToBase64String(btarr);
            }
            else
            {
                //Create xmlSerializer variable
                XmlSerializer xmls = new XmlSerializer(objX.GetType());

                //Create a memory stream
                MemoryStream stream = new MemoryStream();

                xmls.Serialize(stream, objX);

                //return out stream contents
                retVal = GetStringFromStream(stream);
            }

            return retVal;
        }

        /// <summary>
        /// Use this Serialize method if qualified namespace descripters are required in the result xml
        /// </summary>
        /// <param name="objX"></param>
        /// <param name="ns">Dictionary of Namespace Descriptor, Namespace String (i.e. "ns0", "http://myxmlnamespace"</param>
        /// <returns></returns>
        public static string Serialize(object objX, XmlSerializerNamespaces ns)
        {
            string retVal = "";

            //Create xmlSerializer variable
            XmlSerializer xmls = new XmlSerializer(objX.GetType());

            //Create a memory stream
            MemoryStream stream = new MemoryStream();

            xmls.Serialize(stream, objX, ns);

            //return out stream contents
            retVal = GetStringFromStream(stream);

            return retVal;
        }

        /// <summary>
        /// Binary Serializes an object to a byte array
        /// </summary>
        /// <param name="objX">object to be serialized.</param>
        /// <returns>byte Array</returns>
        public static byte[] SerializeToByteArray(object objX)
        {
            // First serialize the object into a memory stream
            MemoryStream stream = new MemoryStream();
            BinaryFormatter objBinaryFormat = new BinaryFormatter();
            objBinaryFormat.Serialize(stream, objX);
            objBinaryFormat = null;

            // Convert the stream into a string
            stream.Position = 0;
            return stream.ToArray();
        }

        /// <summary>
        /// Deserializes a Stream object.
        /// </summary>
        /// <param name="stream">Stream to be deserialized.</param>
        /// <param name="defaultNamespace">Default namespace for all xml elements.</param>
        /// <returns>Deserialized object in the generic type.</returns>
        public static T Deserialize<T>(Stream stream, string defaultNamespace)
        {
            //Create xmlSerializer variable
            XmlSerializer xmls = null;
            if (!string.IsNullOrEmpty(defaultNamespace))
                xmls = new XmlSerializer(typeof(T), defaultNamespace);
            else
                xmls = new XmlSerializer(typeof(T));

            stream.Position = 0;
            return (T) xmls.Deserialize(stream);
        }


        /// <summary>
        /// Deserializes either an xml serialized or binary serialized object.
        /// </summary>
        /// <param name="strObject">Either an xml serialized or binary serialized object.</param>
        /// <param name="binaryString">Flag to determine form of deserialization. 
        /// if ( strObject is a binary string, then this value should be true. 
        /// if ( strObject is an xml string, then this value should be false.</param>
        /// <param name="defaultNamespace">Default namespace for all xml elements</param>
        /// <returns>Deserialized object in the generic type</returns>
        public static T Deserialize<T>(string strObject, bool binaryString, string defaultNamespace)
        {
            if (binaryString)
            {
                MemoryStream memoryStream = null;

                try
                {
                    // convert the string into a stream
                    byte[] btarr = Convert.FromBase64String(strObject);
                    memoryStream = new MemoryStream(btarr);

                    // deserialize the stream into an object graph
                    BinaryFormatter BinaryFormatter = new BinaryFormatter();
                    return (T)BinaryFormatter.Deserialize(memoryStream);
                }
                finally
                {
                    if (memoryStream != null)
                    {
                        memoryStream.Close();
                        memoryStream.Dispose();
                    }
                }
            }
            else
            {
                StringReader stringReader = null;

                try
                {
                    //Create xmlSerializer variable
                    XmlSerializer xmls = null;
                    if (!string.IsNullOrEmpty(defaultNamespace))
                        xmls = new XmlSerializer(typeof(T), defaultNamespace);
                    else
                        xmls = new XmlSerializer(typeof(T));

                    //Create string  Reader
                    stringReader = new StringReader(strObject);

                    //Deserialize object
                    return (T)xmls.Deserialize(stringReader);
                }
                finally
                {
                    if (stringReader != null)
                    {
                        stringReader.Close();
                        stringReader.Dispose();
                    }
                }
            }
        }

        private static string GetStringFromStream(MemoryStream strm1)
        {
            strm1.Position = 0;
            byte[] btarr = strm1.ToArray();
            Encoding Encoder = Encoding.UTF8;
            string retVal = Encoder.GetString(btarr);
            return retVal;
        }
    }
}